# Exercicio 1.

# 1.1.
# b: taxa intrinseca de crescimento da vitima.
# a: eficiencia de captura: efeito de um predador na taxa de crescimento da vitima per capita. 
# e: eficiencia de conversao: capacidade que os predadores tem para converter cada nova vitima em um incremento no crescimento populacional per capita dos predadores.
# s: taxa de mortalidade per capita dos predadores.

# 1.2.
# dV/dt = b * V - a * V^2 + e * a * V^2
# dV/dt = b * V - a * V^2 (1 - e); a(1 - e) = d
# dV/dt = b * V - d * V^2, o que equivale ao modelo logistico, pois:
#       = b * V - b/k * V^2; b/k = d
#       = r * V (1 - V/k); r = b

## Um olhar diferente.
# dV/dt = (b - aP) V
# dp/dt = ea(V + jP) - (s - aj)P,
# onde b, a, ea e s continuam tendo o mesmo significado, j eh a proporcao de canibalismo.


# 2
parametros=c(a=0.01, b=0.5, e=0.1, s=0.2)
inic=c(V0=25,P0=5)
tempo=seq(0,100,by=0.1)

require("deSolve")
ppLV <- function(times,y, parms)
{
  V<-y[1]
  P<-y[2]
  with(as.list(parms),{
    dH.dt <- b* V -a * P * V
    dP.dt <- e * a* P*V -s *P
    return(list(c(dH.dt,dP.dt)))
  })
}
vp.res1=ode(y=inic, times=tempo, func=ppLV, parms=parametros)
matplot(tempo,(vp.res1[,2:3]), type="l", main= "Dinâmica Predador-Presa", ylab="Tamanho da População")

vp.res2=ode(y=c(V0=500,P0=15), times=tempo, func=ppLV, parms=parametros)
vp.res3=ode(y=c(V0=300,P0=50), times=tempo, func=ppLV, parms=parametros)

plot(vp.res1[,2], vp.res1[,3], type="l",main="Dinâmica Predador-Presa",  ylab="Predador", xlab="Presa")
points(25,5 , cex=1.5, pch=19) # definindo no gráfico o estado incial das populações
arrows(x0=c(1300,-20,500), y0=c(125, 175, 0), x1=c(650,-20,950), y1=c(200,100,2), length=0.1)
lines(vp.res2[,2],vp.res2[,3] )
points(500,15,cex=1.5)
lines(vp.res3[,2],vp.res3[,3] )
points(300,50,cex=1.5, pch=2)
abline(h=parametros["b"]/parametros["a"],lty=2)
abline(v=parametros["s"]/(parametros["a"]*parametros["e"]),lty=2)

# 2.1.
# Cada ponto representa os tamanhos iniciais das populacoes de presas e predadores.

# 2.2.
# As linhas solidas representam os ciclos estaveis neutros para cada ponto de partida.

# 2.3.
# As linhas pontilhadas horizontal e vertical representam as isoclinas da presa e do predador respectivamente. 

# 2.4
# A intersecao das isoclinas representa o ponto para no qual nao acontecem ciclos, desde que as populacoes nao sejam afastadas dele, em outras palavras, eh o ponto correspondente a ausencia de crescimento para as duas especies.

# 2.5.
# Abaixo da isoclina horizontal a vitima cresce entanto que acima decresce.A direita da isoclina vertical os predadores aumentam entanto que a esquerda diminuem. Isto faz sentido, pois a iscolina horizontal representa  o numero de predadores (r/v) para controlar a populacao de vitimas, entanto que a isoclina vertical representa o numero de vitimas (q/b) que controla a populacao de predadores. 


# Exercicio 3
orbitang<-function(nsetas=50,inic=c(V0=300,P0=100),parametros=c(a=0.01, b=0.5, e=0.1, s=0.2),fun=ppLV,tmax=100)
{
  tempo=seq(0,tmax,by=0.01)
  vp.res1=ode(y=inic, times=tempo, func=fun, parms=parametros)
  plot((vp.res1[,2]),(vp.res1[,3]), type="l",col=2,ylab="N?mero de predadores", xlab="N?mero de presas",  main= "Din?mica Predador-Presa", xlim=c(0,610), ylim=c(0,150))
  indice<-which(abs(vp.res1[,2]-inic[1])<1)
  indice2<-which(abs(vp.res1[indice,3]-inic[2])<1)
  indice3<-which(indice[indice2]>10)
  periodo<-indice[indice2][indice3][1]
  inter<-as.integer(periodo/nsetas)
  for(i in 1:nsetas)
  {
    points((vp.res1[(inter*i),2]),(vp.res1[(inter*i),3]))
    arrows((vp.res1[(inter*i),2]) , (vp.res1[(inter*i),3]) ,((vp.res1[(inter*i),2])+fun(tempo,(vp.res1[(inter*i),2:3]),parametros)[[1]][1]), ((vp.res1[(inter*i),3])+fun(tempo,(vp.res1[(inter*i),2:3]),parametros)[[1]][2]), length=0.1, lwd=1)
  }
}

LVdiscreto<-function(parametros=c(a=0.01, b=0.5, e=0.1, s=0.2), inic=c(V0=200,P0=60), curvas=40, fun=ppLV,tmax=100,assincrono=FALSE)
{
  cores<-rainbow(curvas)
  plot((parametros["s"]/(parametros["a"]*parametros["e"])),(parametros["b"]/parametros["a"]), main= "Din?mica Predador-Presa",ylab="N?mero de predadores",xlab="N?mero de presas", pch=8, cex=1, xlim=c(0,600), ylim=c(0,150),lwd=2)
  for(i in 1:curvas)
  {
    tempo=seq(0,tmax,by=0.1)
    vp.res1=ode(y=inic, times=tempo, func=fun, parms=parametros)
    for(j in 1:(length(vp.res1[,2])-1))
    {
      lines(c((vp.res1[j,2]),(vp.res1[j+1,2])),c((vp.res1[(j),3]),(vp.res1[(j+1),3])), 		col=cores[i])
    }
    arrows(inic[1],inic[2],(inic[1]+fun(tempo,inic,parametros)[[1]][1]), (inic[2]+fun	(tempo,inic,parametros)[[1]][2]), length=0.1, lwd=2)
    if(assincrono==FALSE)
    {
      pinic1<-(inic[1]+fun(tempo,inic,parametros)[[1]][1])
      pinic2<-(inic[2]+fun(tempo,inic,parametros)[[1]][2])
      inic[1]<-pinic1
      inic[2]<-pinic2
    }
    else
    {
      inic[1]<-(inic[1]+fun(tempo,inic,parametros)[[1]][1])
      inic[2]<-(inic[2]+fun(tempo,inic,parametros)[[1]][2])
    }
  }
}

curvabarulho<-function(times,parms)
{  
  noise1<-parms[round((times/(parms[6]/parms[5])+1))+6]
  noise2<-parms[round((times/(parms[6]/parms[5])+1))+7]
  intert<-(parms[6]/parms[5])
  noise<-noise1+(noise2-noise1)*(times-intert*round(times/intert))/intert
}

LVruido<-function(inic=c(V0=300,P0=100), parametros=c(a=0.01, b=0.5, e=0.1, s=0.2,npicos,tmax,rnorm(npicos,media,variancia)),fun=ppLVr,fabulous=FALSE)
{
  tempo=seq(0,parametros[6],by=0.01)
  vp.res1=ode(y=inic, times=tempo, func=fun, parms=parametros)
  plot((vp.res1[,2]),(vp.res1[,3]), type="l",col=2, main= "Din?mica Predador-Presa", xlim=c(0,610), ylim=c(0,150))
  if (fabulous==TRUE)
  {
    x11()
    plot(1000,1000,xlim=c(0,610), ylim=c(0,150),main= "Din?mica Predador-Presa", 		xlab="N?mero de presas", ylab="N?mero de predadores")
    cores<-rainbow(length(vp.res1[,2]))
    for(j in 1:(length(vp.res1[,2])-1))
    {
      lines(c((vp.res1[j,2]),(vp.res1[j+1,2])),c((vp.res1[(j),3]),(vp.res1[(j+1),3])), col=cores[j])
    }
  }
}

# 1
par(mfrow = c(1, 3))
orbitang(nsetas=50,inic=c(V0=300,P0=100),parametros=c(a=0.01, b=0.5, e=0.1, s=0.2), fun=ppLV)
LVdiscreto(parametros=c(a=0.01, b=0.5, e=0.1, s=0.2), inic=c(V0=300,P0=100), curvas=40, fun=ppLV)
LVdiscreto(parametros=c(a=0.01, b=0.5, e=0.1, s=0.2), inic=c(V0=300,P0=100), curvas=40, fun=ppLV, assincrono=TRUE)

tmax=100
media=0
variancia=0.1
npicos=1000
tmax=1000

parametrosruido=c(a=0.01, b=0.5, e=0.1, s=0.2,npicos,tmax,rnorm(npicos,media,variancia))
plot(1:tmax,curvabarulho(1:tmax,parametrosruido),type="l")
LVruido(inic=c(V0=300,P0=100), parametros=parametrosruido)
LVruido(inic=c(V0=300,P0=100), parametros=parametrosruido, fabulous=TRUE)
par(mfrow = c(1, 1))

# A versao discreta revela a inestabilidade do modelo LV, pois mostra como a cada passo, a orbita desloca-se para fora seguindo a direcao da tangente e eventualmente leva a extincao (pequenas variacoes levam a solucoes inestaveis). Ao incluir assincronia, o modelo torna-se menos inestavel porque as orbitas alem de deslocar-se para fora, podem tambem deslocar-se para dentro (levando a entrecruzamento das orbitas). Mesmo assim, o modelo discreto assincronico tambem termina na extincao.


# Exercicio 4.
# Quando a frequencia dos picos eh baixa, a variabilidade nos ciclos eh tambem baixa, ja que situacao aproxima-se a de um modelo deterministico. A frequencias intermedias apresenta-se a maior variabilidade nos ciclos, possivelmente pela coincidencia temporal entre a frequencia dos ciclos e dos picos. A frequencias elevadas dos picos, os ciclos apresentam menos variabilidade, debido a que as variacoes estocasticas acontecem "tao rapido que efeito prevalecente eh o da amplitude media dos picos".


# Exercicio 5
ppRM=function(t,y,parms)
{
  V<-y[1]
  P<-y[2]
  with(as.list(parms),{
    dV.dt <- b* V * (1-alfa * V) - w * P * V/(D+V)
    dP.dt <- e * w* P*V/(D+V) -s *P
    return(list(c(dV.dt,dP.dt)))
  })
  
} 

b<-0.8
e<-0.07
s<-0.2
w<-5
D=400
alfa<-0.001
K=1/alfa
V<-0:K
Viso<-expression(b/w*(D+(1-alfa *D)*V-alfa*V^2))
VisoEv<-eval(Viso)

param<-c(b=b,alfa=alfa, e=e,s=s,w=w,D=D)
tfinal=150
resPPRM<-ode(y=c(900,120),times=1:tfinal,func=ppRM, parms=param )

plot(V,VisoEv,type="l", ylab="Predador", xlab="Vítima", ylim=c(0,150), col="red", lty=2, main="Modelo Rosenzweing-MacArthur")
abline(v=s*D/(e*w-s), lty=2, col="blue")
arrows(resPPRM[-tfinal,2], resPPRM[-tfinal,3], resPPRM[-1,2], resPPRM[-1,3], length=0.1)

# 5.1.
# Linha vermelha: isoclina do vitima. A forma  de arco pode se dever a um efeito de Alle, a capacidade de suporte da vitima e a resposta funcional do predador entre outros fatores. Se uma populacao de vitimas maior se torna mais eficaz na reproducao, obtencao de recursos ou defesa defesa contra predadores, serao necessarios mais predadores para controla-la,
# Linha azul: isoclina do predador. Esta cruza a isoclina da vitima a direita do topo porque o predador e relativamente ineficiente (alta taxa de mortalidade e baixa taxa de conversao).
# Setas em direcao centripeta: trajetoria que indica equilibrio estavel. O equilibrio estavel deve-se a intersecao das isoclinas a direita do topo da isoclina da vitima.

# 5.2
alfa<-0.0005
K=1/alfa
V<-0:K
Viso<-expression(b/w*(D+(1-alfa *D)*V-alfa*V^2))
VisoEv<-eval(Viso)

param<-c(b=b,alfa=alfa, e=e,s=s,w=w,D=D)
tfinal=150
resPPRM<-ode(y=c(900,120),times=1:tfinal,func=ppRM, parms=param )

plot(V,VisoEv,type="l", ylab="Predador", xlab="Vítima", ylim=c(0,150), xlim=c(0, 2000), col="red", lty=2, main="Modelo Rosenzweing-MacArthur")
abline(v=s*D/(e*w-s), lty=2, col="blue")
arrows(resPPRM[-tfinal,2], resPPRM[-tfinal,3], resPPRM[-1,2], resPPRM[-1,3], length=0.1)

# O aumento de k desestabiliza o sistema (isoclina do predador fica a esquerda do topo da isoclina da pvitima). Isto acontece quando o predador eh relativamente eficiente, baixo a premissa de que a isoclina do predador e vertical. Com tudo, a dinamica predador-vitima atinge um ciclo limite.

# 5.3.

par(mfrow = c(1, 1))
alfa<-0.0005
K=1/alfa
V<-0:K
param<-c(b=b,alfa=alfa, e=e,s=s,w=w,D=D)

LVdiscreto(parametros=param, inic=c(V0=200,P0=80), curvas=60, fun=ppRM, assincrono=TRUE)

# O modelo de (PPRM) mostra-se mais robusto do que o modelo simples de LV e se bem nao eh completamente estavel, eh menos afeitado por variacoes na capacidade de suporte. A diferencia entre o modelo continuo e o discreto eh menos marcada em comparacao com o model LV.

















